/*************************************************************************
 *	This code is written by N. Kehtarnavaz and N. Kim as part of the 
 *	textbook "Real-Time Digital Signal Processing Based on TMS320C6000".
 *************************************************************************/

// Receiver

#define CHIP_6713 1

#include <stdio.h>
#include <c6x.h>
#include <csl.h>
#include <csl_mcbsp.h>
#include <csl_irq.h>

#include "dsk6713.h"
#include "dsk6713_aic23.h"

/*********************************************************
 *  Declarations
 *********************************************************/

#define	AMP		0x4000
#define N ?? // root raised cosine filter length

float output[512];
float samples_sin[N]; //sine samples buffer
float samples_cos[N]; // cosine samples buffer

float sine_table[8]={0,0.7071,1.0000,0.7071,0.0000,-0.7071,-1.0000,-0.7071};
float h[N]={..,..,..}; // root raised cosine filter with oversampling rate
short jindex=0;
int number=0;
DSK6713_AIC23_CodecHandle hCodec;
DSK6713_AIC23_Config config = DSK6713_AIC23_DEFAULTCONFIG; // Codec configuration with default settings

interrupt void serialPortRcvISR(void);

void main()
{
	int	i;

	DSK6713_init();		// Initialize the board support library, must be called first
	hCodec = DSK6713_AIC23_openCodec(0, &config);

	MCBSP_FSETS(SPCR1, RINTM, FRM);
	MCBSP_FSETS(SPCR1, XINTM, FRM);
	MCBSP_FSETS(RCR1, RWDLEN1, 32BIT);
	MCBSP_FSETS(XCR1, XWDLEN1, 32BIT);
	
	DSK6713_AIC23_setFreq(hCodec, DSK6713_AIC23_FREQ_16KHZ); // set the sampling frequency to 16 KHz
		for( i = 0 ; i <= N ; i++ )
		{
		samples_sin[i] = 0;		
		samples_cos[i]=0;
			}
	IRQ_globalDisable();			// Globally disables interrupts
	IRQ_nmiEnable();				// Enables the NMI interrupt
	IRQ_map(IRQ_EVT_RINT1,15);		// Maps an event to a physical interrupt
	IRQ_enable(IRQ_EVT_RINT1);		// Enables the event
	IRQ_globalEnable();				// Globally enables interrupts

  	/*************************************************************************
  	 * Main Loop, wait for Interrupt	    								   *					               
  	 ***************************************************************************/

	while(1);
	{
	}
}

interrupt void receive_ISR()
{
	short j;
	float ttemp,resultsin,resultcos;
	int ftemp;
	int temp,outputsample;
	int temp1;
	
	temp1 = MCBSP_read(DSK6713_AIC23_DATAHANDLE);
	ttemp = (float) ( temp1 >> 16 ) / AMP ;
	


	for( j = N-1; j >= 0; j-- ) 
	{
		samples_sin[j+1] = samples_sin[j];
		samples_cos[j+1] = samples_cos[j];
}

	
	samples_sin[0] = ttemp;  // ???? demodulate the received signal with a sine wave
	samples_cos[0] = ttemp;  // ???? demodulate the received signal with a cosine wave
	
	resultsin=0;
	for (j=0;j<N+1;j++)
	resultsin+=samples_cos[j]*h[j];
	//repeat the lowpass filtering above for cosine demodulated signal 
	//????
	//????
	output[number++] = resultsin;	
	if(number>512)
	number=0;
	
	ftemp = (int) ( - AMP * resultsin );		// Amplify the amplitude and cast data type for sine channel
	temp = (int) ( - AMP * resultcos);;		// Amplify the amplitude and cast data type for cosine channel
	temp = ( ftemp << 16 ) ;
	outputsample = (temp & 0xffff0000)|(ftemp & 0x0000ffff); // Observe both outputs using stereo to mono adaptor connector on the oscilloscope
	MCBSP_write(DSK6713_AIC23_DATAHANDLE, outputsample );

	
}
